local MINIMAPCOUNT = nil
local SOULSHARD = "Soul Shard"
if GetLocale() == "koKR" then
	SOULSHARD = "영혼의 조각"
elseif GetLocale() == "deDE" then
	SOULSHARD = "Seelensplitter"
elseif GetLocale() == "frFR" then
	SOULSHARD = "Fragment d'\195\162me"  
elseif GetLocale() == "zhTW" then
	SOULSHARD = "靈魂裂片"  
end
local TEXTCOLOR = string.format("%02x%02x%02x", 1 * 255, 1 * 255, 1 * 255)
local ICON_SHARD = "Interface\\Icons\\INV_Misc_Gem_Amethyst_02"
local ICON_BLACK = "Interface\\Addons\\eeShards\\black.tga"

local defaults = 
{
	deleteExcess = true,
	maxShards = 30,
	shardAlertOn = false,
	shardAlertCount = 5,
	shortDesc = false,
}


-- Create addon
eeShards = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceHook-2.1", "AceConsole-2.0", "AceDB-2.0", "FuBarPlugin-2.0")
eeShards :RegisterDB("eeShardsDB")


-- AceOption table
eeShards.menu = 
{ 
   type='group',
   args = {
      
      shardAlertOn = {
         type = 'toggle',
         name = "Soul Shard Alert",
         desc = "Alert when low on soul shards",
         get  = function() return eeShards.db.profile.shardAlertOn end,
         set  = function(v) eeShards.db.profile.shardAlertOn = v end,
         order = 1,
      },
         
      shardAlertCount = {
         type = 'range',
         name = "Soul Shard Alert Count",
         desc = "Number of soul shards to alert on",
			get = function() return eeShards.db.profile.shardAlertCount end,
         set = function(v)  eeShards.db.profile.shardAlertCount = v end,
			min = 1,
			max = 20,
			step = 1,
			isPercent = false,
			disabled = function() return not eeShards.db.profile.shardAlertOn end,
         order = 2,
      },
		
		deleteExcess = {
			type = 'toggle',
			name = 'Delete Excess Shards',
			desc = 'Turn on automatic soul shard deletion',
			get = function() return eeShards.db.profile.deleteExcess end,
			set = function(v) eeShards.db.profile.deleteExcess = v end,
			order = 3,
		},
		
		maxShards = {
			type = 'range',
         name = "Maximum Soul Shards",
         desc = "Maximum number of soul shards to hold before we delete",
			get = function() return eeShards.db.profile.maxShards end,
         set = function(v)  eeShards.db.profile.maxShards = v end,
			min = 10,
			max = 60,
			step = 1,
			isPercent = false,
			disabled = function() return not eeShards.db.profile.deleteExcess end,
         order = 4,
		},
		
		shortDesc = {
			type = 'toggle',
			name = 'Short Text Description',
			desc = 'Short Text Description on FuBar panels',
			get = function() return eeShards.db.profile.shortDesc end,
			set = function(v) eeShards.db.profile.shortDesc = v eeShards:Update() end,
			disabled = function() return not FuBar or (eeShards.minimapFrame and eeShards.minimapFrame:IsShown()) end,
			order = 5,
		},
		
		report = {
			type = 'execute',
			name = "Report Soul Shards",
			desc = "Report the number of soul shards",
			func = function() eeShards:ReportSoulshards() end,
			order = 6,
		},
      
   }
}


-- FuBar options
eeShards.name = "eeShards"
eeShards.OnMenuRequest = eeShards.menu
eeShards.hasIcon = true
eeShards.defaultMinimapPosition = 35
eeShards.cannotDetachTooltip = true
eeShards.independentProfile = true
eeShards.defaultPosition = "LEFT"
eeShards.hideWithoutStandby = true
eeShards.blizzardTooltip = true
eeShards.hasNoColor = false



--[[
-- Addon is loaded
--]]
function eeShards:OnInitialize()
	self:Hook(self, "ToggleMinimapAttached","MyMapToggle")
	self:RegisterDefaults('profile', defaults )
	self:RegisterChatCommand( {"/eeShards"}, eeShards.menu )
	if not FuBar then
		self.OnMenuRequest.args.hide.guiName = "Hide minimap icon"
		self.OnMenuRequest.args.hide.desc = "Hide minimap icon"
	end
end



--[[
-- Hooked method for FuBarPlugin:ToggleMinimapAttached()
-- We want to call UpdateText() whenever the icon is attached to the minimap 
-- so the soul shard count will be re-displayed.
--]]
function eeShards:MyMapToggle()
	self:Unhook(self,"ToggleMinimapAttached")
	self:ToggleMinimapAttached()
	self:SetCorrectIcon()
	self:UpdateText()
	self:Hook(self, "ToggleMinimapAttached","MyMapToggle")
end



--[[
-- Start listening for bag events when we've been enabled
--]]
function eeShards:OnEnable()
	self:SetIcon(ICON_SHARD)
	-- Hack: since the minimap icon takes a few seconds to setup, we set a delay
	-- before calling this to find out which icon we really want to show
	self:ScheduleEvent(function() self:SetCorrectIcon() end, 5)
	self:CountSoulshards()
	self:RegisterEvent("BAG_UPDATE",eeShards.BagUpdate)
end



--[[
-- Stop listening for bag events when we've been disabled
--]]
function eeShards:OnDisable()
	-- for god-knows-what reason, this doesn't work, and throws an error.  Disabled for now.
	--self:UnregisterEvent("BAG_UPDATE")
end



--[[
-- Called whenever the bag is updated (item added, moved, removed)
--]]
function eeShards:BagUpdate()
   -- Get the id of the bag updated
	local id = tonumber(arg1)
	
   -- Make sure it's a valid bag
	if not id or id > 4 or id < 0 then
		return
	end
	
   -- Make sure we have the current soul shard count for each bag
	if not eeShards.soulshards then
		eeShards:CountSoulshards()
	end
	
   -- Get the number of soul shards in this bag
	local newamount, empty = eeShards:CountSoulshardsInBag(id)
	
   -- If the number of soul shards hasn't changed for this bag, return
	if newamount == eeShards.soulshards[id] then
		return
	end
	
   -- If the number of soul shards has changed, keep track of the new shard
	local dif = eeShards.soulshards[id] - newamount
	eeShards.totalshards = eeShards.totalshards - dif
	eeShards.soulshards[id] = newamount
	
   -- If soul shard deletion is on, and we're above the limit, delete one
	if eeShards.db.profile.deleteExcess then
		if eeShards.totalshards > eeShards.db.profile.maxShards then
			eeShards:DeleteSoulshards(1,id)
		end
	end
	
   -- If soul shard notification is on, and we're below the limit, notify us
	if eeShards.db.profile.shardAlertOn then
		if eeShards.totalshards <= eeShards.db.profile.shardAlertCount then
			eeShards:Print("LOW ON SHARDS - " .. eeShards.totalshards .. " left")
		end
	end
	
   eeShards:UpdateText()
end



--[[
-- Counts the number of soul shards in all bags
--]]
function eeShards:CountSoulshards()
   -- Variable to hold the number of soul shards for each bag
	self.soulshards = {}
   -- Variable to hold the total number of soul shards
	self.totalshards = 0
	local i,e
   -- Find the number of soul shards in each bag
	for i=0, NUM_BAG_FRAMES do
		self.soulshards[i], e = self:CountSoulshardsInBag(i)
		self.totalshards = self.totalshards + self.soulshards[i]
	end
end



--[[
-- Counts the number of soul shards in a bag
--]]
function eeShards:CountSoulshardsInBag(bag)
	local i
	local ss = 0
	local emptyslots = 0
   -- Loop through every slot in the bag
	for i=1, GetContainerNumSlots(bag) do
		local linktext = GetContainerItemLink(bag, i)
      -- If there's an item, see if it's a soul shard
		if linktext then
			if string.find(linktext, "%["..SOULSHARD.."%]") then
				ss = ss + 1
			end
		else
			emptyslots = emptyslots + 1
		end
	end
   -- Return the number of soul stones and empty slots for this bag
	return ss, emptyslots
end



--[[
-- Attempt to delete a soul shard
--]]
function eeShards:DeleteSoulshards(n, bag)
   -- If no soul shards, or asked to delete 0 soul shards, return
	if self.totalshards <= 0 or n <= 0 then return end
   
   -- if number of soul shards to delete is more than we have..
	if n > self.totalshards then
		n = self.totalshards
	end
	
	local i,j
   -- Number of shards deleted
	local del = 0
   
   -- Check only the supplied bag
	if bag and bag > -1 and bag < 5 then
		for j=1, GetContainerNumSlots(bag) do
			if self:IsItemSoulshard(bag, j) then
				PickupContainerItem(bag, j)
				DeleteCursorItem()
				del = del + 1
				if del == n then return end
			end
		end
   -- Check every bag
	else
		for i=0, NUM_BAG_FRAMES do
			for j=1, GetContainerNumSlots(i) do
				if self:IsItemSoulshard(i, j) then
					PickupContainerItem(i, j)
					DeleteCursorItem()
					del = del + 1
					if del == n then return end
				end
			end
		end
	end
   
end



--[[
-- Determine if an item in a bag is a soul shard
--]]
function eeShards:IsItemSoulshard(bag, slot)
	local linktext = GetContainerItemLink(bag, slot)

	if linktext then
		if string.find(linktext, "%["..SOULSHARD.."%]") then
			return true
		end
	end
	return nil
end



--[[
-- Called by FuBarPlugin regularly
-- We'll use this method to update the soul shard count
--]]
function eeShards:OnTextUpdate()
	-- If we're attached to the minimap, display the soul shard count
	if self.minimapFrame and self.minimapFrame:IsShown() then
		-- Create the font string, if we haven't yet
		if not MINIMAPCOUNT then
			MINIMAPCOUNT = self.minimapFrame:CreateFontString(nil, "ARTWORK")
			MINIMAPCOUNT:SetFontObject(GameFontHighlightSmall)
			MINIMAPCOUNT:SetPoint("CENTER", self.minimapFrame, "CENTER",0,1)
		end
			MINIMAPCOUNT:SetText(self.totalshards)
			MINIMAPCOUNT:Show()
	-- Otherwise, make sure the count text is hidden
	elseif MINIMAPCOUNT and MINIMAPCOUNT:IsVisible() then
		MINIMAPCOUNT:Hide()
	end
   
   -- Update the shard count on the fubar panel
	if self.totalshards and FuBar then
		local s =  string.format("|cff%s%d|r", TEXTCOLOR, self.totalshards)
		if eeShards.db.profile.shortDesc then
			self:SetText(s)
		else
			self:SetText("eeShards:"..s)
		end
	else
		self:SetText("eeShards")
	end
   
end



--[[
-- This will select the icon based on where our addon is loaded
--]]
function eeShards:SetCorrectIcon()
	-- Black background for minimap
	if self:IsMinimapAttached() then
		self:SetIcon(ICON_BLACK)
	-- Soul Shard image for FuBar
	else
		self:SetIcon(ICON_SHARD)
	end
end



--[[
-- Print out the number of soul shards to the chat frame
--]]
function eeShards:ReportSoulshards()
	eeShards:Print(self.totalshards.." Soul Shards")
end
